import { createReducer, on } from '@ngrx/store';

import { UserInfo as RoomUserInfo } from '@ucap/protocol-room';

import * as chattingActions from '../chatting/actions';

import {
  initialState,
  adapterRoom,
  adapterRoomUser,
  RoomUserMap,
  RoomUserShortMap,
  adapterRoomUserShort
} from './state';
import {
  roomsSuccess,
  roomSuccess,
  excludeUser,
  excludeUserSuccess,
  delSuccess,
  rooms2Success
} from './actions';

export const reducer = createReducer(
  initialState,

  on(roomsSuccess, (state, action) => {
    const roomUserMapList: RoomUserMap[] = [];

    for (const key in action.roomUserInfoMap) {
      if (action.roomUserInfoMap.hasOwnProperty(key)) {
        const userInfoList = action.roomUserInfoMap[key];
        if (!!userInfoList && 0 < userInfoList.length) {
          roomUserMapList.push({ roomId: key, userInfos: userInfoList });
        }
      }
    }

    let unreadTotal = 0;

    // tslint:disable-next-line: forin
    for (const key in state.rooms.entities) {
      const value = state.rooms.entities[key];
      unreadTotal += isNaN(value.noReadCnt) ? 0 : value.noReadCnt;
    }
    action.roomList.map((item) => (unreadTotal += item.noReadCnt));

    return {
      ...state,
      rooms: adapterRoom.upsertMany(action.roomList, {
        ...state.rooms,
        syncDate: action.syncDate
      }),
      roomUsers: adapterRoomUser.upsertMany(roomUserMapList, {
        ...state.roomUsers
      }),
      unreadTotal
    };
  }),

  on(rooms2Success, (state, action) => {
    const roomUserMapList: RoomUserMap[] = [];
    const roomUserShortMapList: RoomUserShortMap[] = [];

    for (const key in action.roomUserInfoMap) {
      if (action.roomUserInfoMap.hasOwnProperty(key)) {
        const userInfosMap = action.roomUserInfoMap[key];
        if (!!userInfosMap && 0 < userInfosMap.userInfoList.length) {
          roomUserMapList.push({
            roomId: key,
            userInfos: userInfosMap.userInfoList
          });
        }
        if (!!userInfosMap && 0 < userInfosMap.userInfoShortList.length) {
          roomUserShortMapList.push({
            roomId: key,
            userInfos: userInfosMap.userInfoShortList
          });
        }
      }
    }

    let unreadTotal = 0;

    // tslint:disable-next-line: forin
    for (const key in state.rooms.entities) {
      const value = state.rooms.entities[key];
      unreadTotal += isNaN(value.noReadCnt) ? 0 : value.noReadCnt;
    }
    action.roomList.map((item) => (unreadTotal += item.noReadCnt));

    return {
      ...state,
      rooms: adapterRoom.upsertMany(action.roomList, {
        ...state.rooms,
        syncDate: action.syncDate
      }),
      roomUsers: adapterRoomUser.upsertMany(roomUserMapList, {
        ...state.roomUsers
      }),
      roomUsersShort: adapterRoomUserShort.upsertMany(roomUserShortMapList, {
        ...state.roomUsersShort
      }),
      unreadTotal
    };
  }),

  on(roomSuccess, (state, action) => {
    const roomUserMapList: RoomUserMap[] = [];

    if (!!action.userInfoList && 0 < action.userInfoList.length) {
      roomUserMapList.push({
        roomId: action.roomInfo.roomId,
        userInfos: action.userInfoList
      });
    }

    return {
      ...state,
      rooms: adapterRoom.upsertOne(action.roomInfo, {
        ...state.rooms
      }),
      roomUsers:
        0 < roomUserMapList.length
          ? adapterRoomUser.upsertMany(roomUserMapList, {
              ...state.roomUsers
            })
          : state.roomUsers
    };
  }),

  on(excludeUserSuccess, (state, action) => {
    const roomId = action.roomId;
    const roomUserMap = state.roomUsers.entities[roomId];

    const userInfos: RoomUserInfo[] = [...roomUserMap.userInfos];

    action.userSeqs.forEach((userSeq) => {
      const userInfo: RoomUserInfo = userInfos.find(
        (u) => userSeq === String(u.seq)
      );

      if (!!userInfo && !!userInfo.seq) {
        userInfo.isJoinRoom = false;
      }
    });

    return {
      ...state,
      roomUsers: adapterRoomUser.updateOne(
        {
          id: roomId,
          changes: {
            roomId,
            userInfos
          }
        },
        {
          ...state.roomUsers
        }
      )
    };
  }),

  on(delSuccess, (state, action) => {
    const roomId = action.res.roomId;
    const room = state.rooms.entities[roomId];

    if (!room) {
      return state;
    }

    return {
      ...state,
      rooms: adapterRoom.removeOne(roomId, { ...state.rooms }),
      roomUsers: adapterRoomUser.removeOne(roomId, { ...state.roomUsers })
    };
  })
);
